#version 120

varying vec4 WorldPosition;
varying vec4 pos;
varying vec2 texC;

varying vec3 g; 

varying vec3 frag_position;					// in object space

uniform sampler2D particle_texture;

uniform vec3 light_position[4];				// one light for shading computation
uniform bool light_enable[4];

uniform vec3 eye_position;					// camera position in object space

uniform bool particle_shading_enabled;

// function header
vec4 computeFragColor(vec4 color);
vec3 blinn(vec3 N, vec3 V, vec3 L, int light);

void main()
{
	vec4 texel = texture2D(particle_texture, texC.xy);

	if(particle_shading_enabled)
	{
		gl_FragColor = computeFragColor(texel);
	}
	else
	{
		gl_FragColor = texel;
	}

	if (gl_FragColor.a == 0.0)
	{
		discard;
	}

	//gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);

	//gl_FragColor.a = pos.z / pos.w;
}

vec3 blinn(vec3 N, vec3 V, vec3 L, int light)
{
	vec3 Ka = vec3(1.0, 1.0, 1.0);
	vec3 Kd = vec3(1.0, 1.0, 1.0);
	vec3 Ks = vec3(1.0, 1.0, 1.0);

	float shininess = 100.0;

	float diff_coeff = max(dot(L, N), 0.0);
	
	vec3 H = normalize(L+V);
	
	float spec_coeff = pow(max(dot(H, N), 0.0), shininess);

	if (diff_coeff <= 0.0) 
	{
		spec_coeff = 0.0;
	}

	//return (Ka * gl_LightSource[light].ambient.rgb + Kd * gl_LightSource[light].diffuse.rgb  * diff_coeff + Ks * gl_LightSource[light].specular.rgb * spec_coeff);
	return (Ka + Kd * diff_coeff + Ks * spec_coeff);
}

vec4 computeFragColor(vec4 color)
{
	vec3 grad = g;

	float gradient_magnitude = length(grad);

	vec3 N = normalize(grad);

	vec3 V = normalize(eye_position - frag_position);

	vec3 shaded_color = color.rgb;

	//Only one light source
	for(int i=0; i<1; i++)
	{
		//if (light_enable[i])
		{
			vec3 L = normalize(light_position[i] - frag_position);

			if (dot(L, N) < 0.0)
			{
				L = -L;
			}

			shaded_color = shaded_color + shaded_color * blinn(N, V, L, i);
		}
	}

	vec4 return_color = vec4(shaded_color, color.a);

	return return_color;
}