#version 150 
#extension GL_EXT_gpu_shader4 : enable
#extension GL_EXT_geometry_shader4 : enable

layout (points) in;
layout (triangle_strip, max_vertices = 8) out;

precision highp float;

uniform sampler3D gradient;

uniform int width;
uniform int height;
uniform int depth;

uniform mat4 gl_ProjectionMatrix;
uniform mat4 gl_ModelViewMatrix;

uniform vec3 gradMin;
uniform vec3 gradMax;

uniform float particle_size;

varying vec3 g; 

out vec2 texC;

void main(void)
{
	vec3 texCoords = (gl_PositionIn[0].xyz + 10.0) / 20.0;

	int w = 1;
	g = vec3(0.0);
	for (int i = -w; i < w; ++i) {
		for (int j = -w; j < w; ++j) {
			for (int k = -w; k < w; ++k) {
				g += texture(gradient, texCoords.xyz + vec3(k/width, j/height, i/depth)).xyz * (gradMax - gradMin) + gradMin;
			}
		}
	}

	g = texture(gradient, texCoords.xyz).xyz * (gradMax - gradMin) + gradMin;

	//g = g + g1xx + g0xx + gx1x + gx0x + gxx1 + gxx0;
	g = normalize(g);

	// need to compute one vector orthogonal to the matrix
	// i.e. one solution to x' gradient = 0

	vec3 t1;
	if (dot(g, t1) >= 0.9) {
		t1 = vec3(0.0, 1.0, 0.0);
	} else {
		t1 = vec3(1.0, 0.0, 0.0);
	}
	vec3 t2 = normalize(cross(t1, g));
	t1 = normalize(cross(t2, g));

	// now we have an cartesian coordinate system spanned by gradient, t1, 
	// t2 (hurray!!)

	// now we need to get our 4 points and multiply it w/ our 
	// view + projection matrix to get the final result

	mat3 m = mat3(t1, t2, g);
	//m = mat3(1,0,0,0,1,0,0,0,1);
	mat4 t = mat4(1,0,0,0,0,1,0,0,0,0,1,0, gl_PositionIn[0]);

	// point 1: 0.5 0.5 0.0
	texC.st = vec2(1.0, 1.0);
	gl_Position = gl_ProjectionMatrix * gl_ModelViewMatrix * t * vec4(m * vec3(particle_size, particle_size, 0.0), 1);
	EmitVertex();

	// point 2: -0.5 0.5 0.0
	texC.st = vec2(0.0, 1.0);
	gl_Position = gl_ProjectionMatrix * gl_ModelViewMatrix * t * vec4(m * vec3(-particle_size, particle_size, 0.0), 1);
	EmitVertex();

	// point 3: 0.5 -0.5 0.0
	texC.st = vec2(1.0, 0.0);
	gl_Position = gl_ProjectionMatrix * gl_ModelViewMatrix * t * vec4(m * vec3(particle_size, -particle_size, 0.0), 1);
	EmitVertex();

	// point 4: -0.5 -0.5 0.0
	texC.st = vec2(0.0, 0.0);
	gl_Position = gl_ProjectionMatrix * gl_ModelViewMatrix * t * vec4(m * vec3(-particle_size, -particle_size, 0.0), 1);
	EmitVertex();

	EndPrimitive();

	// backsite
	// point 1: 0.5 0.5 0.0
	/*
	texC.st = vec2(1.0, 1.0);
	gl_Position = gl_ProjectionMatrix * gl_ModelViewMatrix * t * vec4(m * vec3(particle_size, particle_size, 0.01), 1);
	EmitVertex();

	// point 3: 0.5 -0.5 0.0
	texC.st = vec2(0.0, 1.0);
	gl_Position = gl_ProjectionMatrix * gl_ModelViewMatrix * t * vec4(m * vec3(particle_size, -particle_size, 0.01), 1);
	EmitVertex();

	// point 2: -0.5 0.5 0.0
	texC.st = vec2(1.0, 0.0);
	gl_Position = gl_ProjectionMatrix * gl_ModelViewMatrix * t * vec4(m * vec3(-particle_size, particle_size, 0.01), 1);
	EmitVertex();

	// point 4: -0.5 -0.5 0.0
	texC.st = vec2(0.0, 0.0);
	gl_Position = gl_ProjectionMatrix * gl_ModelViewMatrix * t * vec4(m * vec3(-particle_size, -particle_size, 0.01), 1);
	EmitVertex();

	EndPrimitive();*/
}
